/*
 *  logging.hpp
 *
 *  Log messages to stdout.
 *
 *  Joris van Rantwijk 2024
 */

#ifndef PUZZLEFW_LOGGING_H_
#define PUZZLEFW_LOGGING_H_

#include <stdarg.h>
#include <stdio.h>
#include <time.h>


namespace puzzlefw {


enum log_severity { LOG_INFO = 1, LOG_ERROR };


void log(log_severity severity, const char *format, ...)
    __attribute__ ((format (printf, 2, 3)));


void log(log_severity severity, const char *format, ...)
{
    va_list ap;
    va_start(ap, format);

    struct timespec tp;
    clock_gettime(CLOCK_REALTIME, &tp);

    char datestr[80];
    struct tm tlocal;
    strftime(datestr, sizeof(datestr),
             "%Y-%m-%d %H:%M:%S",
             localtime_r(&tp.tv_sec, &tlocal));

    const char *severity_str = (severity == LOG_INFO) ? "INFO " : "ERROR";

    char buf[800];
    vsnprintf(buf, sizeof(buf), format, ap);

    printf("%s.%03d (%s) %s\n",
           datestr,
           (int)(tp.tv_nsec / 1000000),
           severity_str,
           buf);
    fflush(stdout);

    va_end(ap);
}


}  // namespace puzzlefw

#endif  // PUZZLEFW_LOGGING_H_
